#include "stdafx.h"
#include <sstream>
#include <memory>
#include <boost/format.hpp>
#include "Control.h"
#include "NFilePSDK.h"
#include "Utility.h"
#include "NFileStdio.h"

using namespace std;

CMurasameDlg* Control::m_pDialog = 0;
bool Control::m_bStop = false;

void Control::SetDialog(CMurasameDlg& dlg)
{
	m_pDialog = &dlg;
}

const int iBufferSize = 16 * 1024 * 1024;

UINT Control::StartDivide(LPVOID param)
{
	m_bStop = false;
	const string file_path = m_pDialog->GetEditString(EDIT_DIV_FILE_PATH);
	if (!Utility::IsFile(file_path)){
		return -1;
	}

	m_pDialog->EnableWindow(FALSE);
	CListBox& list_box = m_pDialog->GetListBox(LIST_DIV);
	NFilePSDK file_from(file_path, GENERIC_READ, FILE_SHARE_READ, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN);
	if (!file_from.IsOpen()){
		m_pDialog->SetStateText("~");
		m_pDialog->EnableWindow(TRUE);
		return -1;
	}
	long long total_file_size = 0;
	int count = list_box.GetCount();
	for (int i = 0; i != count; ++i){
		CString buffer;
		list_box.GetText(i, buffer);
		long long offset, size;
		istringstream((LPCTSTR)buffer) >> offset >> size;
		size -= offset;
		total_file_size += size;
	}
	long long total_process_size = 0;
	auto_ptr<char> buffer(new char[iBufferSize]);
	for (int index = 0; index != count && !m_bStop; ++index){
		CString buffer_list;
		list_box.GetText(index, buffer_list);
		long long offset, size;
		istringstream((LPCTSTR)buffer_list) >> offset >> size;
		size -= offset;
		const string file_name_to = Utility::GenerateBackupFileName(file_path);
		NFilePSDK file_to(file_name_to, GENERIC_WRITE, 0, CREATE_ALWAYS, FILE_FLAG_WRITE_THROUGH);
		if (!file_to.IsOpen()){
			m_pDialog->SetStateText(string("t@C̍쐬Ɏs : ") + file_name_to);
			m_pDialog->EnableWindow(TRUE);
			return -1;
		}
		file_from.Seek(offset, NFile::SeekBegin);
		long long file_offset = 0;
		while (file_offset != size && !m_bStop){
			m_pDialog->SetStateText((boost::format("Ǎ %1%") % file_name_to).str());
			m_pDialog->SetProgress((float)(total_process_size) / (float)(total_file_size * 2));
			int read_size = file_from.Read(buffer.get(), (int)std::min<long long>(size - file_offset, iBufferSize));
			total_process_size += read_size;

			m_pDialog->SetStateText((boost::format(" %1%") % file_name_to).str());
			m_pDialog->SetProgress((float)(total_process_size) / (float)(total_file_size * 2));
			file_offset += read_size;
			file_to.Write(buffer.get(), read_size);
			file_to.Flush();
			total_process_size += read_size;
		}
	}
	m_pDialog->SetStateText("I");
	m_pDialog->SetProgress(1.0f);
	m_pDialog->EnableWindow(TRUE);
	return 0;
}

UINT Control::StartCombine(LPVOID param)
{
	m_pDialog->EnableWindow(FALSE);
	m_bStop = false;
	CListBox& list_box = m_pDialog->GetListBox(LIST_COMBINE_FILE_PATH);
	int count = list_box.GetCount();
	
	//NFilePSDK file_to(m_pDialog->GetEditString(EDIT_COMBINE_FILE_PATH), GENERIC_WRITE, 0, CREATE_ALWAYS, FILE_ATTRIBUTE_TEMPORARY);
	NFilePSDK file_to(m_pDialog->GetEditString(EDIT_COMBINE_FILE_PATH), GENERIC_WRITE, 0, CREATE_ALWAYS, FILE_FLAG_WRITE_THROUGH);
	//NFileStdio file_to(m_pDialog->GetEditString(EDIT_COMBINE_FILE_PATH), "wbT");
	if (!file_to.IsOpen()){
		m_pDialog->SetStateText("~");
		m_pDialog->EnableWindow(TRUE);
		return -1;
	}
	//file_to.SetBuffer(16 * 1024 * 1024);
	long long total_file_size = 0;
	for (int index = 0; index != count; ++index){
		char file_name_buffer[MAX_PATH];
		list_box.GetText(index, file_name_buffer);
		total_file_size += Utility::GetFileSize(file_name_buffer);
	}
	long long total_process_size = 0;
	auto_ptr<char> buffer(new char[iBufferSize]);
	for (int index = 0; index != count && !m_bStop; ++index){
		char file_name_buffer[MAX_PATH];
		list_box.GetText(index, file_name_buffer);
		const string file_name = file_name_buffer;
		//NFileStdio file_from(file_name_buffer, "rbS");
		NFilePSDK file_from(file_name_buffer, GENERIC_READ, FILE_SHARE_READ, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN);
		//file_from.SetBuffer(4 * 1024);
		long long file_size = Utility::GetFileSize(file_name);
		long long file_offset = 0;
		while (file_offset != file_size && !m_bStop){
			m_pDialog->SetStateText((boost::format("Ǎ %1%") % string(file_name_buffer)).str());
			m_pDialog->SetProgress((float)(total_process_size) / (float)(total_file_size * 2));
			int read_size = file_from.Read(buffer.get(), iBufferSize);
			total_process_size += read_size;

			m_pDialog->SetStateText((boost::format(" %1%") % string(file_name_buffer)).str());
			m_pDialog->SetProgress((float)(total_process_size) / (float)(total_file_size * 2));
			file_offset += read_size;
			file_to.Write(buffer.get(), read_size);
			file_to.Flush();
			total_process_size += read_size;
		}
	}
	m_pDialog->SetStateText("I");
	m_pDialog->SetProgress(1.0f);
	m_pDialog->EnableWindow(TRUE);
	return 0;
}

void Control::Stop()
{
	m_bStop = true;
}
