#include "stdafx.h"
#include <algorithm>
#include <fstream>
#include <sstream>
#include "SettingFile.h"
#include "Utility.h"

using namespace std;

bool SettingFile::Load(const std::string& file_path)
{
	ifstream ifs(file_path.c_str());
	if (!ifs.is_open()){
		return false;
	}
	string last_section;
	string in;
	while (getline(ifs, in)){
		if (in.empty()){
			continue;
		}
		if (in.find("//") == 0){
			continue;
		}
		if (in[0] == '['){
			last_section = in.substr(1, in.size() - 2);
			m_vecSection.push_back(last_section);
		} else {
			istringstream iss(in);
			string key, value;
			getline(iss, key, '=');
			getline(iss, value);
			Add(last_section, key, value);
		}
	}
	sort(m_vecSection.begin(), m_vecSection.end());
	m_vecSection.erase(unique(m_vecSection.begin(), m_vecSection.end()), m_vecSection.end());
	return true;
}

bool SettingFile::Save(const std::string& file_path)const
{
	ofstream ofs(file_path.c_str());
	if (!ofs.is_open()){
		return false;
	}
	for (map<string, map<string, string> >::const_iterator it_section = m_Table.begin(); it_section != m_Table.end(); ++it_section){
		ofs << "[" << it_section->first << "]" << endl;
		for (map<string, string>::const_iterator it_key = it_section->second.begin(); it_key != it_section->second.end(); ++it_key){
			ofs << it_key->first << "=" << it_key->second << endl;
		}
	}
	return true;
}

const std::vector<std::string> SettingFile::GetKeys(const std::string& section)const
{
	vector<string> res;
	map<string, map<string, string> >::const_iterator it_section = m_Table.find(section);
	if (it_section == m_Table.end()){
		return res;
	}
	res.reserve(it_section->second.size());
	for (map<string, string>::const_iterator it_key = it_section->second.begin(); it_key != it_section->second.end(); ++it_key){
		res.push_back(it_key->first);
	}
	return res;
}

const string strEmpty("");

const std::string& SettingFile::Get(const std::string& section, const std::string& key)const
{
	map<string, map<string, string> >::const_iterator it_section = m_Table.find(section);
	if (it_section == m_Table.end()){
		return strEmpty;
	}
	map<string, string>::const_iterator it_key = it_section->second.find(key);
	if (it_key == it_section->second.end()){
		return strEmpty;
	}
	return it_key->second;
}

void SettingFile::Add(const std::string& section, const std::string& key, const std::string& value)
{
	m_Table[section][key] = value;
}

long long SettingFile::GetInt(const std::string& section, const std::string& key)const
{
	const std::string str = Get(section, key);
	if (str.empty()){
		return INT_MAX;
	}
	return Utility::ToInt(str);
}

void SettingFile::AddInt(const std::string& section, const std::string& key, long long value)
{
	Add(section, key, Utility::ToString(value, 0));
}

bool SettingFile::IsInSection(const std::string& section)const
{
	return m_Table.find(section) != m_Table.end();
}

bool SettingFile::IsInKey(const std::string& section, const std::string& key)const
{
	map<string, map<string, string> >::const_iterator it_section = m_Table.find(section);
	if (it_section == m_Table.end()){
		return false;
	}
	return it_section->second.find(key) != it_section->second.end();
}
